#!/usr/bin/env python

from __future__ import division
from math import sin, cos, sqrt
from cmath import exp
import os
import time
import readline
import string
gnuplot = os.popen("gnuplot", "w")

def frange(tmin, tmax, tstep):
    count = int((tmax - tmin)/tstep)
    for i in range(count):
        t = tmin + i*tstep
        yield t
def fourierTransform(f, tmin, tmax, dt):
    # F{f}[w] = int dt f[t] exp[i w t]
    #return lambda w: sum([f[t]*cos(w*t) for t in frange(tmin, tmax, dt)])
    def g(w, t):
        return f[t]*exp(w*t*1j)
    return lambda w: abs(sum([g(w, t) for t in frange(tmin, tmax, dt)]) - dt*g(w, tmin)/2 - dt*g(w, tmax)/2)
def discretize(f, tmin, tmax, tstep):
    return [(t, f(t)) for t in frange(tmin, tmax, tstep)]
#def discreteFourierTransform(f, tmin, tmax, dt):
#    fd = fourierTransform(f, tmin, tmax, dt)
#   return discretize(fd, tmin, tmax, dt)
class Vector(object):
    def __init__(self, *elements):
        self.data = elements
    def __iter__(self):
        return iter(self.data)
    def __getitem__(self, index):
        return self.data[index]
    def __add__(self, b1):
        return Vector(*tuple((a + b) for a, b in zip(self, b1)))
    def __len__(self):
        return len(self.data)
    def __rmul__(self, b):
        return Vector(*tuple((a*b) for a in self))
    def __str__(self):
        return str(self.data)
def run(driver, mass, theta0, Omega, dampening, g, length, dt, count):
    """ 
    z = (t, theta, theta')^T
    dz/dt = F(z[t])
    Runge-Kutta-Naeherung:
    """
    omega0 = sqrt(g/length)
    eta = dampening/(2*mass)
    def F(z):
        return Vector(1, z[2], driver*sin(Omega*z[0]) - 2*eta*z[2] - omega0**2*sin(z[1]))
    def stepper(k1, k2, k3, k4):
        return 1/6*(k1 + 2*k2 + 2*k3 + k4)
    z = Vector(0, theta0, 0)
    for i in range(count):
        t = dt*i
        z = Vector(t, z[1], z[2])
        yield z
        k1 = F(z)
        k2 = F(z + dt/2*k1)
        k3 = F(z + dt/2*k2)
        k4 = F(z + dt*k3)
        z = z + dt*stepper(k1, k2, k3, k4)
def writeIntoFile(values, fname):
    try:
        fo = open(fname, "w")
        for t, z in values:
             fo.write("%s %s\n" % (t, z))
        fo.close()
    except:
        import traceback
        traceback.print_stack()
        pass
def writePhaseSpaceIntoFile(values, fname):
    try:
        fo = open(fname, "w")
        for z in values:
             fo.write("%s %s\n" % (z[1], z[2])) # t, z[1]
        fo.close()
    except:
        import traceback
        traceback.print_stack()
        pass
if __name__ == "__main__":
    class S(object):
        driver=0
        mass=1E-9
        theta0=2
        Omega=2
        dampening=0
        #omega0=2
        g=9.80665#m/s
        dt=0.1
        length=5E-3#m
        count=200
        maxfrequency=20
        dfrequency=0.01
    def printSettings(S):
            for name in dir(S):
                if not name.startswith("_"):
                    print name, "=", getattr(S, name)
    while True:
        line = raw_input(">>> ")
        if line.find("=") > -1:
            name, value = map(string.strip, line.split("=", 1))
            value = eval(value, dict([(name, getattr(S, name)) for name in dir(S)]))
            setattr(S, name, value)
        printSettings(S)
        values = [z for z in run(driver=S.driver, mass=S.mass, theta0=S.theta0, Omega=S.Omega, dampening=S.dampening, length=S.length, g=S.g, dt=S.dt, count=S.count + 1)]
        writePhaseSpaceIntoFile(values, "L")
        #[t, z[1] for t, z in values]
        spectrum = [(z[0], z[1]) for z in discretize(fourierTransform(dict([(z[0], z[1]) for z in values]), 0, S.count*S.dt, S.dt), 0, S.maxfrequency, S.dfrequency)]
        writeIntoFile(spectrum, "S")
        gnuplot.write("plot 'L' w l\n")
        gnuplot.write("pause -1\n\n")
        gnuplot.flush()
    time.sleep(10)

