! useful types: COMPLEX*16, DOUBLE PRECISION, INTEGER
! useful conversion functions: DCMPLX(a, b), DBLE(z), CONJ(z), ABS(z)
! arrays: v(0:999)
! open arrays: v(0:*)
PROGRAM a
  IMPLICIT NONE
  DOUBLE PRECISION :: tmin
  DOUBLE PRECISION :: tmax
  DOUBLE PRECISION :: theta0
  DOUBLE PRECISION :: dampening
  DOUBLE PRECISION :: g
  DOUBLE PRECISION :: driver
  DOUBLE PRECISION :: length
  DOUBLE PRECISION :: mass
  ! ??? , intent(in) :: result
  DOUBLE PRECISION :: dt
  DOUBLE PRECISION :: Omega
  DOUBLE PRECISION :: values(0:999, 0:2)
  DOUBLE PRECISION :: dfrequency
  DOUBLE PRECISION :: maxfrequency
  driver = 0D0
  mass = 1D-9
  theta0 = 2
  Omega = 2
  dampening = 0
  g = 9.80665 ! m/(s*s)
  dt = 0.1
  length = 5D-3
  tmin = 0D0
  tmax = 20D0
  maxfrequency = 200
  dfrequency = 0.01
  CALL run(driver, mass, theta0, Omega, dampening, g, length, tmin, tmax, dt, values)
  CALL writePhaseSpaceIntoFile(values)
  ! TODO spectrum = [(z[0], z[1]) for z in discretize(fourierTransform(dict([(z[0], z[1]) for z in values]), 0, S.count*S.dt, S.dt), 0, S.maxfrequency, S.dfrequency)]
  !CALL writeSpectrumIntoFile(spectrum)
END
COMPLEX*16 FUNCTION g(f, tmin, tmax, dt, w, i) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION, intent(in) :: tmax
  DOUBLE PRECISION, intent(in) :: tmin
  DOUBLE PRECISION, intent(in) :: dt
  !DOUBLE PRECISION, intent(in) :: t
  INTEGER i
  DOUBLE PRECISION, intent(in) :: w
  DOUBLE PRECISION, intent(in) :: f(0:999,0:2)
  result = f(1,i)*EXP(DCMPLX(0,w*(tmin + i*dt)))
END
! always transforms z(1)
FUNCTION fourierTransform(f, tmin, tmax, dt, wmin, wmax, wstep) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION, intent(in) :: tmax
  DOUBLE PRECISION, intent(in) :: tmin
  DOUBLE PRECISION, intent(in) :: dt
  DOUBLE PRECISION, intent(in) :: wmax
  DOUBLE PRECISION, intent(in) :: wmin
  DOUBLE PRECISION, intent(in) :: wstep
  DOUBLE PRECISION, intent(in) :: f(0:999,0:2)
  COMPLEX*16, dimension(0:999) :: result
  COMPLEX*16 :: g
  COMPLEX*16 r
  INTEGER wcount
  INTEGER tcount
  INTEGER wi
  INTEGER ti
  DOUBLE PRECISION t
  DOUBLE PRECISION w
  wcount = (wmax - wmin)/wstep
  tcount = (tmax - tmin)/dt
  DO wi = 0,wcount - 1
    w = wmin + wi*wstep
    r = 0D0
    DO ti = 0,tcount - 1
      t = tmin + ti*dt
      r = r + g(f, tmin, tmax, dt, w, ti)
    END DO
    result(wi) = abs(r - dt*g(f, tmin, tmax, dt, w, 0)/2 - dt*g(f, tmin, tmax, dt, w, tcount)/2)
  END DO
END
FUNCTION stepper(k1, k2, k3, k4) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION, intent(in) :: k3(0:2)
  DOUBLE PRECISION, intent(in) :: k2(0:2)
  DOUBLE PRECISION, intent(in) :: k1(0:2)
  DOUBLE PRECISION, intent(in) :: k4(0:2)
  DOUBLE PRECISION, dimension(0:2) :: result
  INTEGER I
  DO I = 0, 2
    result(I) = 1/6*(k1(I) + 2*k2(I) + 2*k3(I) + k4(I))
  END DO
END
FUNCTION FF(driver, Omega, eta, omega0, z) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION, intent(in) :: Omega
  DOUBLE PRECISION, intent(in) :: z(0:2)
  DOUBLE PRECISION, dimension(0:2) :: result
  DOUBLE PRECISION, intent(in) :: driver
  DOUBLE PRECISION, intent(in) :: eta
  DOUBLE PRECISION, intent(in) :: omega0
  result(0) = 1
  result(1) = z(2)
  result(2) = driver*sin(Omega*z(0)) - 2*eta*z(2) - omega0**2*sin(z(1))
END
SUBROUTINE run(driver, mass, theta0, Omega, dampening, g, length, tmin, tmax, dt, result)
  IMPLICIT NONE
  INTERFACE
    FUNCTION FF(driver, Omega, eta, omega0, z)
      DOUBLE PRECISION, intent(in) :: Omega
      DOUBLE PRECISION, intent(in) :: z(0:2)
      DOUBLE PRECISION, dimension(0:2) :: FF
      DOUBLE PRECISION, intent(in) :: driver
      DOUBLE PRECISION, intent(in) :: eta
      DOUBLE PRECISION, intent(in) :: omega0
    END FUNCTION
    FUNCTION stepper(k1, k2, k3, k4)
      DOUBLE PRECISION, intent(in) :: k3(0:2)
      DOUBLE PRECISION, intent(in) :: k2(0:2)
      DOUBLE PRECISION, intent(in) :: k1(0:2)
      DOUBLE PRECISION, intent(in) :: k4(0:2)
      DOUBLE PRECISION, dimension(0:2) :: stepper
    END FUNCTION
  END INTERFACE
  DOUBLE PRECISION, intent(in) :: tmin
  DOUBLE PRECISION, intent(in) :: tmax
  DOUBLE PRECISION, intent(in) :: theta0
  DOUBLE PRECISION, intent(in) :: dampening
  DOUBLE PRECISION, intent(in) :: g
  DOUBLE PRECISION, intent(in) :: driver
  DOUBLE PRECISION, intent(in) :: length
  DOUBLE PRECISION, intent(in) :: mass
  DOUBLE PRECISION, dimension(0:999, 0:2) :: result
  DOUBLE PRECISION, intent(in) :: dt
  DOUBLE PRECISION, intent(in) :: Omega
  DOUBLE PRECISION :: z(0:2)
  DOUBLE PRECISION :: ss(0:2)
  DOUBLE PRECISION :: eta
  INTEGER tcount
  INTEGER i
  INTEGER j
  DOUBLE PRECISION, dimension(0:2) :: k1
  DOUBLE PRECISION, dimension(0:2) :: k2
  DOUBLE PRECISION, dimension(0:2) :: k3
  DOUBLE PRECISION, dimension(0:2) :: k4
  DOUBLE PRECISION, dimension(0:2) :: k2a
  DOUBLE PRECISION, dimension(0:2) :: k3a
  DOUBLE PRECISION, dimension(0:2) :: k4a
  DOUBLE PRECISION t
  DOUBLE PRECISION :: omega0
  ! " \n    z = (t, theta, theta')^T\n    dz/dt = FF(z[t])\n    Runge-Kutta-Naeherung:\n    "
  omega0 = sqrt(g/length)
  WRITE(*,*) "omega0 waere", omega0
  eta = dampening/(2*mass)
  z(0) = 0D0
  z(1) = theta0
  z(2) = 0D0
  tcount = (tmax - tmin)/dt
  DO i = 0, tcount - 1
    t = tmin + dt*i
    z(0) = t
    DO j = 0, 2
      result(i,j) = z(j)
    END DO
    k1 = FF(driver, Omega, eta, omega0, z)
    DO j = 0, 2 
      k2a(j) = z(j) + dt/2*k1(j)
    END DO
    k2 = FF(driver, Omega, eta, omega0, k2a)
    DO j = 0, 2
      k3a(j) = z(j) + dt/2*k2(j)
    END DO
    k3 = FF(driver, Omega, eta, omega0, k3a)
    DO j = 0, 2
      k4a(j) = z(j) + dt*k3(j)
    END DO
    k4 = FF(driver, Omega, eta, omega0, k4a)
    ss = stepper(k1, k2, k3, k4)
    DO j = 0, 2
      z(j) = z(j) + dt*ss(j)
    END DO
  END DO
END
SUBROUTINE writeSpectrumIntoFile(values)
  IMPLICIT NONE
  COMPLEX*16, intent(in) :: values(0:999, 0:2)
  INTEGER I
  OPEN(9, FILE="S", STATUS="REPLACE")
  DO I = 0,999
    write(9,*) values(I,0), values(I,1)
  END DO
  CLOSE(9)
END
SUBROUTINE writePhaseSpaceIntoFile(values)
  IMPLICIT NONE
  DOUBLE PRECISION, intent(in) :: values(0:999, 0:2)
  INTEGER I
  OPEN(9, FILE="L", STATUS="REPLACE")
  DO I = 0,999
    write(9,*) values(I,1), values(I,2)
  END DO
  CLOSE(9)
END

