#!/usr/bin/env python

from __future__ import division
from math import sin

class Vector(object):
    def __init__(self, *elements):
        self.data = elements
    def __iter__(self):
        return iter(self.data)
    def __getitem__(self, index):
        return self.data[index]
    def __add__(self, b1):
        return Vector(*tuple((a + b) for a, b in zip(self, b1)))
    def __len__(self):
        return len(self.data)
    def __rmul__(self, b):
        return Vector(*tuple((a*b) for a in self))
    def __str__(self):
        return str(self.data)
def run(f, theta0, Omega, dampening, omega0, dt):
    """ 
    z = (t, theta, theta')^T
    dz/dt = F(z[t])
    Runge-Kutta-Naeherung:
    """
    def F(z):
        return Vector(1, z[2], f*sin(Omega*z[0]) - 2*dampening*z[2] - omega0**2*sin(z[1]))
    def stepper(k1, k2, k3, k4):
        return 1/6*(k1 + 2*k2 + 2*k3 + k4)
    z = Vector(0, theta0, 0)
    for i in range(200):
        t = dt*i
        yield t, z
        k1 = F(z)
        k2 = F(z + dt/2*k1)
        k3 = F(z + dt/2*k2)
        k4 = F(z + dt*k3)
        z = z + dt*stepper(k1, k2, k3, k4)
if __name__ == "__main__":
    for t, z in run(f=0, theta0=2, Omega=2, dampening=0, omega0=2, dt=0.1):
         print z[1], z[2] # t, z[1]

