MODULE globals
  integer Lmax,NN                  !/* Maximal lattice size, # neighbors*/
  parameter (Lmax=200,NN=4)
  integer Inn(NN)                    !/* Nearest neighbor array I */
  parameter (Inn=(/1,-1,0,0/))
  integer Jnn(NN)                       !/* Nearest neighbor array J */
  parameter (Jnn= (/0,0,1,-1/)) 
  real JJ, HH                      !/* Coupling constant, magnetic field */
  parameter (JJ=1.d0,HH=0d0)
END
Program Ising
  use globals
  implicit none
  integer sweeps                    !/* number of measurement sweeps */
  integer warm                      !/* number of warm-up sweeps */
  integer L                             !/* lattice dimension */
 ! /***************************
 !  * Initialization          *
 !  ***************************/
 ! write (*,*) "Gittergroesse (LxL) ?"
 ! read(*,*) L
  !write (*,*) "# sweeps?"
  !read(*,*) sweeps
  !write (*,*) "# warm up sweeps?"
  !read(*,*) warm
  sweeps = 100000
  warm = 10000
  L = 10
  CALL printGraph(L, sweeps, warm)
  WRITE(*,*) ""
  WRITE(*,*) ""
  L = 15
  CALL printGraph(L, sweeps, warm)
  WRITE(*,*) ""
  WRITE(*,*) ""
  L = 30
  CALL printGraph(L, sweeps, warm)
end Program Ising

subroutine printGraph(L, sweeps, warm)
  use globals
  implicit none
  integer L, sweeps, warm
  integer mag(Lmax,Lmax)            !/* 2D Ising Lattice */
  real*8 M
  real*8 M2
  real*8 M4
  real*8 E
  INTEGER Ti
  real*8 T                          !/* temperature (in units of J/k_B) */
  ! T, <|M|>, <M**2>, <M**4>, (1 - M4/(3*M2**2)), E
  DO Ti = 0,200
      T = 1D0 + Ti*5D0/200
      CALL calculateM124E(mag,L, M, M2, M4, E, T, sweeps, warm)
      write (*,*) T, M, M2, M4, (1D0 - M4/(3D0*(M2**2D0))), E
  END DO
END

subroutine calculateM124E(mag,L, M, M2, M4, E, T, sweeps, warm)
  use globals
  implicit none
  real*8 sumall
  real CalcE
  real*8 Es, dummyE, Ms, M2s, M4s
  integer iwarm, n, L, sweeps, warm
  integer mag(Lmax,Lmax)
  real*8 M, M2, M4, E, T, beta, mags
  beta = 1D0/T
  call random_init(mag,L,Lmax)  
  Es = CalcE(mag,L)
  Ms = 0D0
  M2s = 0D0
  M4s = 0D0
  dummyE = 0D0
! /* warum up sweeps */
  DO iwarm = 1, warm
    call simulateSweep(mag,L,beta,dummyE) 
  END DO
! /* end warum up sweeps */
! /*  sweeps */
  DO n = 1, sweeps - warm
    call simulateSweep(mag,L,beta,Es)
    mags = sumall(mag, L)
    Ms = Ms + abs(mags)
    M2s = M2s + mags**2
    M4s = M4s + mags**4
  END DO
! /*  end sweeps */
  !call  outputmag(mag,L)
  M = Ms/DBLE(sweeps)/DBLE(L**2) ! -> intensive g.
  M2 = M2s/DBLE(sweeps)/DBLE(L**4) ! -> intensive g.
  M4 = M4s/DBLE(sweeps)/DBLE(L**8) ! -> intensive g.
  E = Es/DBLE(sweeps)
end
subroutine random_init(mag,L,Lmax)
  implicit none
  integer Lmax
  integer mag(Lmax,Lmax), L
 
  integer i,j
  
  do i=1,L
     do j=1,L
        if (rand().gt.0.5d0) then
           mag(i,j) = 1
        else
           mag(i,j) = -1
        endif
     enddo
  enddo

  return
end subroutine random_init

subroutine outputmag(mag,L,Lmax)
  integer Lmax
  integer mag(Lmax,Lmax), L
 
  integer i,j
  write (*,*) 'Spin snapshot'

  do i=1,L
     do j=1,L
        if(mag(i,j).eq. 1) then
           write(*,'(a)',ADVANCE='NO') '-> '
        else
           write(*,'(a)',ADVANCE='NO') '<- '
        endif
     enddo
     !     Newline to complete
     write (*,*) 
     write (*,*)
  enddo
  return 
end subroutine outputmag




real function CalcE(mag,L)
  use globals
  implicit none
  integer mag(Lmax,Lmax),L
  
  integer i,j,k,Inew,Jnew
  real*8 Energy

  !/* Determine the initial energy */
  Energy = 0.0
  
  do i=1,L
     do j=1,L
        
        !/* Loop over nearest neighbors */
        do k=1, NN  
            Inew = i + Inn(k)       
            Jnew = j + Jnn(k)
    
            !/* Check periodic boundary conditions */
            if (Inew .le. 0) then
              Inew = L
            else 
               if(Inew .gt. L) then 
                  Inew = 1
               endif
            endif    
            if (Jnew .le. 0) then
              Jnew = L
            else 
               if(Jnew .gt. L) then 
                  Jnew = 1
               endif
            endif
            
            !/* Update the energy */
            Energy = Energy-JJ * mag(i,j) * mag(Inew,Jnew)
        enddo
        !/*Calculate the contribution from the field H */
        Energy = Energy - 2.d0*HH*mag(i,j);
     enddo
   enddo

   !/* Account for double counting */
   Energy = Energy/2.d0;

   CalcE=Energy
   return
end function CalcE
INTEGER function sumNearestNeighbours(mag, L, i, j) result(result)
        use globals
        implicit none
        integer mag(Lmax,Lmax),L
        integer i,j,k,Inew,Jnew
        result = 0
        do k=1, NN  
            Inew = i + Inn(k)       
            Jnew = j + Jnn(k)
    
            !/* Check periodic boundary conditions */
            if (Inew .le. 0) then
              Inew = L
            else 
               if(Inew .gt. L) then 
                  Inew = 1
               endif
            endif    
            if (Jnew .le. 0) then
              Jnew = L
            else 
               if(Jnew .gt. L) then 
                  Jnew = 1
               endif
            endif
      result = result + mag(inew,jnew)
    end do
end
real*8 function sumall(mag, L) result(result)
    use globals
    implicit none
    integer mag(Lmax,Lmax), L
    integer i,j
    result = 0D0
    do i = 1, L
      do j = 1, L
        result = result + mag(i,j)
      end do
    end do
end
subroutine simulateSweep(mag,L,beta,Es)
 use globals
 implicit none
 integer mag(Lmax,Lmax),L
 real*8 beta
 real rn
 integer i,j,k,k2
 real*8 deltaE
 integer sumNearestNeighbours 
 real*8 Es ! IO
 do k2=1, L**2
  k = rand()*L
  i = 1 + k
  k = rand()*L
  j = 1 + k
  deltaE = 2d0*JJ*mag(i,j)*(sumNearestNeighbours(mag, L, i, j)) + 2d0*mag(i,j)*HH
  rn = rand()
  if (rn < EXP(-beta*deltaE)) then
    mag(i,j) = -mag(i,j)
    Es = Es + deltaE
  end if
 end do
 ! 1 sweep fertig
end subroutine

