/**********************************************************************/
/* life.c                                                             */
/*                                                                    */
/* Monte-Carlo Demo                                                   */
/* Die Population des Modells besteht aus Bauern(gelb,40%),           */
/* Handwerkern(rot, 20%), Jaegern (gelb,30%) und Kriegern(blau, 10%). */
/* Je 50% sind maennlich und weiblich. Ihr Lebensraum besteht aus     */
/* 100x100 Feldern. Anfangs werden 1000 Individuen statistisch        */
/* generiert und auf die Felder aufgeteilt.                           */
/* Die Lebensregeln sind:                                             */
/* Krieger wandern weit und toeten eventuelle Bewohner des neuen Felds*/
/* Jaeger wandern etwas weniger weit, Handwerker und Bauern noch      */
/* weniger. Wenn die auf Felder des anderen Geschlechts kommen,       */
/* entstehen neue Individuen nach obiger Wahrscheinlichkeit, Bauern   */
/* haben viele Nachkommen. Jede Wanderung ist mit Altern verbunden,   */
/* hohes Alter=Tod. Im Sueden ist die Lebenswerwartung viel hoeher    */
/* als im Norden.                                                     */
/*                                                                    */
/* Durch Variation der Parameter kommen Szenarios, in denen die       */
/* Population waechst, in anderen stirbt sie aus (Balkengrafik)       */
/* (Schwarze Balken=Gesamtpopulation, bunte Balken=Anteil der Typen)  */
/* Mit den eingestellten Parametern waechst sie.                      */
/**********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <plot.h>
#include <math.h>
#define Num 1000
//
//struct individuum
//
struct individuum 
{long age;
 enum Gender{noG,male,female} gender; //noG, noP = undefiniert
 enum Profession{noP,Handwerk,Jagd,Krieg,Landwirtschaft} profession;
};
//
// counter fuer Individuen
// Alle, Jagd, Handwerk, Krieg, Landwirtschaft
//
int counter[5]={0,0,0,0,0};
//
// myrand()
//
long myrand(long n)
{long r;
 do
 {r=rand();
 }while(r>(RAND_MAX/n)*n);
 return r%(n+1);
}
//
// myfrand()
//
double myfrand(double n)
{return n*(double)rand()/((double)RAND_MAX+1.);
}
//
// updatecounter
//
void updatecounter(struct individuum in,int i)
{int sum;
 if(in.profession==Handwerk)       
 {counter[1]+=i;
 }
 else if(in.profession==Landwirtschaft) 
 {counter[2]+=i;
 }
 else if(in.profession==Jagd) 
 {counter[3]+=i;
 }
 else if(in.profession==Krieg)
 {counter[4]+=i;
 }
 counter[0]=10000-counter[1]-counter[2]-counter[3]-counter[4];
}
//
// showarray()
// Zeichnet Anfangszustand
void showarray(struct individuum in[100][100])
{int i,j;
 pl_colorname ("yellow");
 pl_filltype(1);
 for(i=0;i<100;++i)
 for(j=0;j<100;++j)
 {if(in[i][j].age==0)                            pl_colorname("white");
  else if (in[i][j].profession==Handwerk)        pl_colorname ("red");
  else if (in[i][j].profession==Krieg)           pl_colorname ("blue");
  else if (in[i][j].profession==Landwirtschaft)  pl_colorname ("yellow");
  else if (in[i][j].profession==Jagd)            pl_colorname ("green");
  pl_box(i,j,i+1,j+1);
 }
}
//
// showcounter()
// Bargrafik
void showcounter()
{int sum,pop;
 static int rep=0;
 sum=counter[1]+counter[2]+counter[3]+counter[4];
 pop=10000-counter[0];
 pl_colorname("black");  pl_box(120,0,129,(100*pop)/10000);
 pl_colorname("red"  );  pl_box(130,0,139,(100*counter[1])/sum);
 pl_colorname("yellow"); pl_box(140,0,149,(100*counter[2])/sum);
 pl_colorname("green");  pl_box(150,0,159,(100*counter[3])/sum);
 pl_colorname("blue" );  pl_box(160,0,169,(100*counter[4])/sum);

/*  pl_colorname("white");  pl_box(120,(100*pop)/10000+1,129,100); */
 pl_colorname("white");  pl_box(130,(100*counter[1])/sum+1,139,100);
 pl_colorname("white");  pl_box(140,(100*counter[2])/sum+1,149,100);
 pl_colorname("white");  pl_box(150,(100*counter[3])/sum+1,159,100);
 pl_colorname("white");  pl_box(160,(100*counter[4])/sum+1,179,100);
 rep++;
 pl_colorname("black");
 pl_point(rep/200,-200+sum/50);
 if(rep/200>=200)
 {pl_colorname("white");
  pl_box(0,-200,200,0);
  rep=0;
 }
}
//
// showindividuum()
// Zeichnet laufende updates.
void showindividuum(struct individuum in,int i,int j)
{pl_filltype(1);
 if(in.age==0)                            pl_colorname("white");
 else if (in.profession==Handwerk)        pl_colorname ("red");
 else if (in.profession==Krieg)           pl_colorname ("blue");
 else if (in.profession==Landwirtschaft)  pl_colorname ("yellow");
 else if (in.profession==Jagd)            pl_colorname ("green");
 pl_box(i,j,i+1,j+1);
}
//
// moveidividuum()
// verschiebt individuum.
void moveindividuum(struct individuum *in1,struct individuum *in2)
{*in2=*in1;
 in1->age=0;
 in1->profession=noP;
 in1->gender=noG;
}
//
// newindividuum
// 
void newindividuum(struct individuum* in)
{float probMale          = 0.50; //Wahrscheinlichkeiten
 float probFemale        = 0.50;
 float probJagd          = 0.10;
 float probHandwerk      = 0.20;
 float probKrieg         = 0.30;
 float probLandwirtschaft= 0.40;
 double r;
 if(in->age) return;
 if(myfrand(1.)>probMale) in->gender=male; 
 else in->gender=female;
 r=myfrand(1.); 
 if     (r<probJagd)                        in->profession=Jagd;
 else if(r<probJagd+probHandwerk)           in->profession=Handwerk;
 else if(r<probJagd+probHandwerk+probKrieg) in->profession=Krieg;
 else                                       in->profession=Landwirtschaft;
 
 in->age=1; //Anfangsalter
 updatecounter(*in,1);
 return;
}
//
// killindividuum()
//
void killindividuum(struct individuum* in)
{if(in->profession==noP) return;
 updatecounter(*in,-1);
 in->gender=noG;
 in->profession=noP;
 in->age=0;
}
//
// life()
//
void life()
{int i,i1,j,j1,k,N,cnt;
 float r;
 struct individuum in[100][100] ;
 
 for(i=0;i<100;i++)
 for(j=0;j<100;j++)
 killindividuum(&in[i][j]); //Initialisiert alle Felder
 
 for(k=0;k<Num;k++)
 {i=myrand((long)99);
  j=myrand((long)99);
  newindividuum(&in[i][j]); //Anfangspopulation
 }
 showarray(in);
 while(1)                   //Wird sich selbst ueberlassen 
// for(k=0;k<100000;k++)
 {i=myrand((long)99);                                   //Wuerfle ein Feld i,j
  j=myrand((long)99);
  if(( in[i][j].age>1 && j>60) || (in[i][j].age>20))    //Individuum stirbt im Sueden mit 21
                                                        //und im Norden (j>60) mit 2 dn
  {killindividuum(&in[i][j]);                           //Alterstod
   showindividuum(in[i][j],i,j); 
  }
  if(in[i][j].age>0) in[i][j].age+=1;                   //Jede(r) selektierte altert
  if(in[i][j].profession==Krieg)                         //Krieger(in)  
  {i1=i+myrand(10)-5; if(i1<0) i1+=99; if(i1>99) i1-=99; //Sprung um bis zu 10 Felder
   j1=j+myrand(10)-5; if(j1<0) j1+=99; if(j1>99) j1-=99; //negativ = Umrundung 
   in[i][j].age+=1;
   killindividuum(&in[i][j]);                            //geh hin, toete
   showindividuum(in[i][j],i,j); 
   showindividuum(in[i1][j1],i1,j1); 
  }
  else if(in[i][j].profession==Jagd)                    //Jaeger(in)
  {i1=i+myrand(6)-3; if(i1<0) i1+=99; if(i1>99) i1-=99; //Sprung um bis zu 6 Felder 
   j1=j+myrand(6)-3; if(j1<0) j1+=99; if(j1>99) j1-=99;
   if(in[i1][j1].age==0)                                //wenn niemand dort geh hin 
   {moveindividuum(&in[i][j],&in[i1][j1]);
    showindividuum(in[i][j],i,j); 
    showindividuum(in[i1][j1],i1,j1); 
   }
   else  if(in[i1][j1].gender!= in[i][j].gender)        //opposite sex dort
   {i1=myrand((long)99);
    j1=myrand((long)99);
    if(in[i1][j1].age==0)                               //geh nicht hin, aber erzeuge
    {newindividuum(&in[i1][j1]);                        //neues Individuum irgendwo  
     showindividuum(in[i1][j1],i1,j1); 
  }}}
  else if(in[i][j].profession==Landwirtschaft || in[i][j].profession==Handwerk)
  {i1=i+myrand(2)-1; if(i1<0) i1+=99; if(i1>99) i1-=99; //Sprung um bis zu 2 Felder 
   j1=j+myrand(2)-1; if(j1<0) j1+=99; if(j1>99) j1-=99;
   if(in[i1][j1].age==0)                                //niemand dort, geh hin 
   {moveindividuum(&in[i][j],&in[i1][j1]);
    showindividuum(in[i][j],i,j); 
    showindividuum(in[i1][j1],i1,j1); 
   }
   else if(in[i1][j1].gender!=in[i][j].gender)          //opposite sex dort
   {int ii;                                             //geh nicht hin aber erzeuge
    for(ii=0;ii<5;++ii)                                 //5 neue Individuen, irgendwo
    {i1=myrand((long)99);
     j1=myrand((long)99);
     if(in[i1][j1].age==0)
     {newindividuum(&in[i1][j1]);
      showindividuum(in[i1][j1],i1,j1); 
  }}}}
// showcounter();
  pl_flushpl();
  if(cnt++>100)
  {cnt=0;
   showcounter();
  }
 }
}
//
// main()
//
int main()
{int i, handle;
 srand(time(0));
 pl_parampl ("BITMAPSIZE", "1000x1000"); 
 handle = pl_newpl ("X", stdin, stdout, stderr);
 pl_selectpl (handle);         /* select the Plotter for use */
    
 pl_openpl();                  /* begin page of graphics */
 pl_space (0, -200, 200, 100);    /* specify user coordinate system */

 life();

 pl_closepl ();                /* end page of graphics */
 pl_selectpl (0);              /* select default Plotter */
 pl_deletepl (handle);         /* delete Plotter we used */
 return 0;
}





