/**********************************************************************/
/* rand1a.c                                                           */
/*                                                                    */
/* Aufruf:                                                            */
/* rand1a Anzahl Typ                                                  */
/*                                                                    */
/* Berechnet gleichverteilte Zufallszahlen und stellt sie             */
/* grafisch dar. Statt rand() sollen hier versuchsweise andere/eigene */
/* Algorithmwn verwendet werden.                                      */
/* Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])    */
/*                   Negativer Wert: zeichnet Kreise statt Punkte     */
/*                   (manchmal besser sichtbar)                       */
/* Typ    = (int),   einer der folgenden Generatoren:                 */
/*  0 ... Quotient des least significant Bytes von rand() und 255     */
/*  1 ... Quotient der least significant 10 bits von rand() und 1023  */
/*  2 ... Methode von Park und Miller (Numerical Recipies)            */
/*  3 ... Lehmer Methode mit a=ran(), b=16807, m=RAND_MAX             */
/*        [Overflow-Gefahr!!]                                         */
/*  4 ... Lehmer Methode mit a=211, b=1663, m=7865                    */
/*  Beispiel: rand1a 50000 1                                          */
/*                                                                    */
/* Compilieren: gcc rand1a.c -o rand1a -lplot -lm                     */
/*                                                                    */
/**********************************************************************/

// Verschiedene Zudallszahlengeneratoren
// Achtung: Erzeugte Zufallszahlen sind hier float
#include <stdio.h>
#include <stdlib.h>
#include <plot.h>
#include <math.h>
#include <time.h>

float myrand0()
{long help1, help2;
 help1=rand();
 help2=RAND_MAX;
 help1&=255; // least significant byte
 help2=255;
 return (float)help1/(float)help2;
}
float myrand1()
{long help1, help2;
 help1=rand();
 help2=RAND_MAX;
 help1&=1023; // least significant 10 bits
 help2=1023;
 return (float)help1/(float)help2;

}

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define MASK 123459876
float myrand2()
/* Algorithmus von Park und Miller, Siehe Numerical Recipies */
/* Kein overflow trotz grossem m (hier: IM) */
{static long idum;
 static int init=0;
 long k;
 float ans;
 if(!init) 
 {init++;
  srand(time(0)); 
  idum=rand();
 }
 idum ^= MASK;
 k=(idum)/IQ;
 idum=IA*(idum-k*IQ)-IR*k; 
 if (idum < 0) idum += IM;
 ans=AM*idum;
 idum ^= MASK; 
 return ans;
}

float myrand3()
{static long idum;
 static int init;
 float ans;
 if(!init) 
 {init++;
  idum=RAND_MAX/2;
 }
 idum=(16807*idum) % 2147483647;    // LONG_MAX == 2147483647
/* Achtung: (16807*idum) kann den erlaubten integer-Bereich ueberschreiten
Moeglicherweise berechnet der floating-point Prozessor das Zwischenergebnis
trotzdem korrekt. Das Ergebnis ist daher implementationsabhaengig */  
 ans=(float)idum/(float)RAND_MAX; // RAND_MAX == LONG_MAX 
 return ans;
}

float myrand4()
{static long idum;
 static int init;
 float ans;
 if(!init) 
 {init++;
  idum=211;
 }
 idum=(1663*idum) % 7865;
 ans=(float)idum/7865.;
 return ans;
}

void draw_randomdots (int n, int ff)
{int handle, i;
 float x,y;
 float (*f[5])()={myrand0, myrand1, myrand2, myrand3, myrand4};
 if(ff<0 || ff>4) {printf("\nUngueltiger Parameter 2 (0,1,2,3,4)\n"); exit(-1);}
 handle=pl_newpl("X",stdin,stdout,stderr);
 pl_selectpl(handle);
 pl_openpl();
 pl_fspace(0.,0.,1., 1.);
 if(n>0)
 for(i=0;i<n;++i)
 {x=f[ff]();
  y=f[ff]();
  pl_fpoint(x,y);
 }
 else 
 for(n=-n,i=0;i<n;++i)
 {x=f[ff]();
  y=f[ff]();
  pl_fcircle(x,y,.005);
 }
 pl_closepl();
}

void usage()
{printf("\nAufruf:\n"
 "rand1a Anzahl Type\n"
 "Berechnet gleichverteilte Zufallszahlen und stellt sie grafisch dar.\n"
 "Statt rand() sollen hier versuchsweise eigene Algorithmen verwendet werden.\n"
 "Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])\n"
 "         Negativer Wert: zeichnet Kreise statt Punkte (manchmal besser sichtbar)\n"
 "Typ    = (int),   einer der folgenden Generatoren:\n"
 " 0 ... Quotient des least significant Bytes von rand() und 255\n"
 " 1 ... Quotient der least significant 10 bits von rand() und 1023\n"
 " 2 ... Methode von Park und Miller (Numerical Recipies)\n"
 " 3 ... Lehmer Methode mit a=ran(), b=16807, m=RAND_MAX [Overflow-Gefahr!!]\n"
 " 4 ... Lehmer Methode mit a=211, b=1663, m=7865\n"
 "Beispiel: rand1 50000 1\n"
 "Die Grafik wird durch Anklicken geloescht.\n\n");
 exit(-1);
}
int main (int argc, char** argv)
{int n;
 if(argc!=3) usage();
 draw_randomdots(atoi(argv[1]),atoi(argv[2]));
}





