/**********************************************************************/
/* rand2a.c                                                           */
/*                                                                    */
/* Unterschied zu rand1.c: grafische Darstellung durch Striche        */
/* (Photonenpfade) und zu exponentialverteilten Zeitpunkten           */
/* (Simulation der statistischen Zeitintervalle der Photonenemission) */
/* Fuer die Wartezeit wird die Funktion usleep() verwendet            */
/*                                                                    */
/* Aufruf:                                                            */
/* rand2a Anzahl Absorptionskoeffizient                               */
/*                                                                    */
/* Berechnet log-verteilte Zufallszahlen und stellt sie               */
/* grafisch dar.                                                      */
/* Es entspricht das entstehende Bild dem von Photonen, die von       */
/* oben auf eine Probe auftreffen und verschieden weit eindringen     */
/* Die Erklaerung des zugrundeliegenden Absorptionsgesetzes im        */
/* Zusammenhang mit desem Beispiel finde Sie im Skriptum              */
/*                                                                    */
/* Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])    */
/* Beispiel: rand2 100 10.                                            */
/* (usleep() = langsam -> kleine Photonzahl whlen)                   */
/*                                                                    */
/* Fuer die Grafiken wird die GNU plotlib mit float-Funktionen        */
/* verwendet                                                          */
/*                                                                    */
/* Compilieren: gcc rand2a.c -o rand2a -lplot -lm                     */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <plot.h>
#include <math.h>

void timerand()
{static double tsum=0;
 static int isum=0;
 double timer=(double)rand()/(double)RAND_MAX;
 double timeconstant=.1;
 timer=-timeconstant*log(timer);
 usleep((unsigned)(timer*1000000.)); //microseconds
 return;
}

void draw_randomdots (int n, double mu)
{int handle,i;
 float x,y;
 handle=pl_newpl("X",stdin,stdout,stderr);
 pl_selectpl(handle);
 pl_openpl();
 pl_fspace(0.,0.,1.,1.);
 for(i=0;i<n;++i)
 {x=(double)rand()/(double)RAND_MAX; // In x-Richtung gleichverteilt
  y=1.-(-(1./mu)*log((double)rand()/(double)RAND_MAX)); 
  pl_fline(x,1.,x,y);
  timerand();
 }
 pl_closepl();
}
void usage()
{printf("\nAufruf:\n"
 "rand2 Anzahl Absorptionskoeffizient\n"
 "Berechnet log-verteilte Zufallszahlen und stellt sie grafisch dar.\n"
 "Im Gegensatz zu rand2 als Striche und mit zufaelliger Zeitfolge\n"
 "(Anwendbar auf die Absorption von Photonen entlang eines Wegs)\n"
 "Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])\n"
 "Absorptionskoeffizient = (float) in der Einheit 1/cm\n" 
 "Beispiel: rand2a 100 10.\n"
 "Die Grafik wird durch Anklicken geloescht.\n\n");
 exit(-1);
}

int main (int argc, char** argv)
{int n;
 if(argc!=3) usage();
 draw_randomdots(atoi(argv[1]), atof(argv[2]));
}

