/**********************************************************************/
/* rand3.c                                                            */
/*                                                                    */
/* Aufruf:                                                            */
/* rand3 Anzahl Typ                                                   */
/*                                                                    */
/* Berechnet verschieden verteilte Zufallszahlen und stellt sie       */
/* grafisch dar (gleichverteilt, linear, Gauss, zweidimensional-Gauss */
/*               exponential(time)                                    */
/* Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])    */
/* Typ    = (char*), Auswahl: const lin gauss gaussgauss time         */
/* Beispiel: rand1 50000 gaussgauss                                   */
/* Die Zeitkonstante fuer die Option time ist 0.7s                    */
/*                                                                    */
/* Compilieren: gcc rand3.c -o rand3 -lplot -lm                       */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <plot.h>
#include <math.h>
#include <time.h>
//#include <curses.h>

void RandomDistr(double* x,double* y, char* type,int n)
{static int init=0;
 int seed=time(NULL);
 if(!init++) srand(seed);

 if(!strcmp(type,"const"))
 {*x=(double)rand()/(double)RAND_MAX;
  *y=(double)rand()/(double)RAND_MAX;
  return;
 }
 else if(!strcmp(type,"lin"))
 {*x=pow((double)rand()/(double)RAND_MAX,0.5);
  *y=(double)rand()/(double)RAND_MAX;
  return;
 }
 else if(!strcmp(type,"gaussgauss"))
 {double help;
  do
  {do
   {*x=2.*(double)rand()/(double)RAND_MAX-1.;
    *y=2.*(double)rand()/(double)RAND_MAX-1.;
    help=*x**x+*y**y;
   }while(help>=1 || help==0);
   help=sqrt(-2.0*log(help)/help);
   *x=0.5+*x*help/8;
   *y=0.5+*y*help/8;
  }while(*x<0. || *x>=1. || *y<0 || *y>=1.);
  return;
 }
 else if(!strcmp(type,"gauss"))
 {double help;
  do
  {do
   {*x=2.*(double)rand()/(double)RAND_MAX-1.;
    *y=2.*(double)rand()/(double)RAND_MAX-1.;
    help=*x**x+*y**y;
   }while(help>=1 || help==0);
   help=sqrt(-2.0*log(help)/help);
   *x=0.5+*x*help/8;
   *y=(double)rand()/(double)RAND_MAX;
  }while(*x<0. || *x>=1. || *y<0 || *y>=1.);
  return;
 }
 else if(!strcmp(type,"time"))
 {static double tsum=0;
  static int isum=0;
  double timer=(double)rand()/(double)RAND_MAX;
  double timeconstant=.7;
  timer=-timeconstant*log(timer);
  if(n<100)
  {usleep((unsigned)(timer*1000000.)); //microseconds
   printf("\a");
  }
  isum++;
  tsum+=timer;
  *x=(double)isum/(double)n;
  *y=tsum/isum;
  if(*y>1.) *y=1.;
  printf("this=%lf total=%lf average=%lf\n", timer, tsum, tsum/isum);
  fflush(stdout);
  return;
 }
}

void draw_randomdots (int n, char* type)
{int handle,i;
 double x,y;
 handle=pl_newpl("X",stdin,stdout,stderr);
 pl_selectpl(handle);
 pl_openpl();
 pl_fspace(0.,0.,1.,1.);
 for(i=0;i<n;++i)
 {RandomDistr(&x,&y,type,n);
//  pl_fpoint(x,y);
  pl_fcircle(x,y,.002);
 }
 pl_closepl();
}

void usage()
{printf("\nAufruf:\n"
 "rand3 Anzahl Typ\n"
 "Berechnet Zufallszahlen und stellt sie grafisch dar.\n"
 "Anzahl = (int),   Anzahl der Datenpunkte (Zufallszahlen[paare])\n"
 "Typ    = (char*), Auswahl: const lin gauss gaussgauss time\n"
 "Beispiel: rand3 5000 gaussgauss\n"
 "Die Zeitkonstante fuer die Option \"time\" ist 0.7 s\n" 
 "Die Grafik wird durch Anklicken geloescht.\n\n");
 exit(-1);
}

int main (int argc, char** argv)
{if(argc!=3) usage();
 srand(time(0));
 draw_randomdots(atoi(argv[1]), argv[2]);
}









