PROGRAM schroedingergleichung
  IMPLICIT NONE
  COMPLEX*16 psi0
  DOUBLE PRECISION V
  DOUBLE PRECISION dx
  DOUBLE PRECISION x
  INTEGER m ! x step index
  INTEGER n ! t step index
  DOUBLE PRECISION xmin
  DOUBLE PRECISION xmax
  DOUBLE PRECISION xrange
  DOUBLE PRECISION dt
  DOUBLE PRECISION t
  DOUBLE PRECISION tmin
  DOUBLE PRECISION tmax
  DOUBLE PRECISION trange
  DOUBLE PRECISION tau
  COMPLEX*16 diagUpper
  COMPLEX*16 diagMain(0:999)
  COMPLEX*16 diagLower
  COMMON diagUpper, diagMain, diagLower
  INTEGER xresolution
  INTEGER tresolution
  PARAMETER (xmin = -22D0)
  PARAMETER (xmax = 32D0)
  PARAMETER (xrange = xmax - xmin)
  PARAMETER (xresolution = 80)
  PARAMETER (tmin = 0D0)
  PARAMETER (tmax = 140D0)
  PARAMETER (trange = tmax - tmin)
  PARAMETER (tresolution = 140)
  COMPLEX*16 psixs(0:xresolution - 1)
  PARAMETER (dt = trange/tresolution)
  PARAMETER (dx = xrange/xresolution)
  PARAMETER (tau = dt/(dx*dx))
  COMPLEX*16 inhomogenity(0:xresolution - 1)
  COMPLEX*16 solveLinearSystem
  DOUBLE PRECISION length
  DOUBLE PRECISION psiNorm
  ! calculate psi for starting time
  DO m = 0,xresolution - 1,1
    x = xmin + m*dx
    psixs(m) = psi0(x)
  END DO
  psixs(0) = CMPLX(0.0D0, 0.0D0)
  psixs(xresolution - 1) = CMPLX(0.0D0, 0.0D0)
  DO m = 0,xresolution - 1,1
    x = xmin + m*dx
    WRITE(*,*) 0.0D0, x, REAL(CONJG(psixs(m))*psixs(m)) ! print amplitudes
  END DO
  WRITE(*,*) ""
  WRITE(*,*) ""
  ! do time evolution
  DO n = 1,tresolution - 1,1
    t = tmin + n*dt
    ! calculate inhomogenity
    DO m = 0,xresolution - 1,1
      x = xmin + m*dx
      inhomogenity(m) = psixs(m) 
          END DO
    ! calculate psi for next time moment
    ! solve (1 + I tau A + I dt V) psi2 = psi, for psi2
    diagUpper = DCMPLX(0D0, -tau*0.5D0)
    DO m = 0,xresolution - 1,1
      x = xmin + m*dx
      diagMain(m) = DCMPLX(1D0, tau*(1D0) + dt*V(x))
    END DO
    diagLower = DCMPLX(0D0, -tau*0.5D0)
    DO m = 0,xresolution - 1,1
      x = xmin + m*dx
      psixs(m) = solveLinearSystem(xresolution, inhomogenity, m)
      WRITE(*,*) t, x, REAL(CONJG(psixs(m))*psixs(m)) ! print amplitudes
    END DO
    psixs(0) = CMPLX(0.0D0, 0.0D0)
    psixs(xresolution - 1) = CMPLX(0.0D0, 0.0D0)
    length = sqrt(psiNorm(dx, psixs, xresolution))
    DO m = 0,xresolution - 1,1
      psixs(m) = psixs(m)/length
    END DO
    WRITE(*,*) ""
    WRITE(*,*) ""
  END DO
END
DOUBLE PRECISION FUNCTION V(x) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION x
  IF (10D0 < x .AND. x < 14D0) THEN
    result = 10D0
  ELSE
    result = 0D0
  END IF
END
COMPLEX*16 FUNCTION psi0(x) RESULT (result)
  IMPLICIT NONE
  DOUBLE PRECISION x
  DOUBLE PRECISION sigma
  DOUBLE PRECISION k0
  DOUBLE PRECISION pi
  PARAMETER (pi = 3.141592653589793D0)
  sigma = 2.0D0
  k0 = 1.0D0
  result = 1.0D0/(SQRT(sigma*SQRT(pi)))*exp(-x**2/(2*sigma**2))*exp(DCMPLX(0,1)*k0*x)
  !WRITE(*,*) "psi0", x, result
END

DOUBLE PRECISION FUNCTION psiNorm(dx, psixs, N) RESULT (result)
  IMPLICIT NONE
  INTEGER N
  DOUBLE PRECISION dx
  COMPLEX*16 psixs(0:*)
  INTEGER m
  result = 0D0
  DO m = 0,N - 1
    result = result + dx*CONJG(psixs(m))*psixs(m)
  END DO
END
! solve tridiagonal matrices

RECURSIVE COMPLEX*16 FUNCTION y(inhomogenity, I) RESULT (RESULT)
  IMPLICIT NONE
  INTEGER I
  COMPLEX*16 inhomogenity(0:*)
  COMPLEX*16 lambda
  IF (I .EQ. 0) THEN
    RESULT = inhomogenity(I)
  ELSE
    RESULT = inhomogenity(I) - lambda(I)*y(inhomogenity, I - 1)
  END IF
END
RECURSIVE COMPLEX*16 FUNCTION solveLinearSystem(N, inhomogenity, I) RESULT (RESULT) ! OK
  IMPLICIT NONE
  INTEGER I
  COMPLEX*16 d
  COMPLEX*16 y
  COMPLEX*16 inhomogenity(0:*)
  COMPLEX*16 diagUpper
  COMPLEX*16 diagMain(0:999)
  COMPLEX*16 diagLower
  COMMON diagUpper, diagMain, diagLower
  INTEGER N
  IF (I .EQ. N - 1) THEN
    RESULT = y(inhomogenity, I)/d(I)
  ELSE
    RESULT = (y(inhomogenity, I) - diagUpper*solveLinearSystem(N, inhomogenity, I + 1))/d(I)
  END IF
END
RECURSIVE COMPLEX*16 FUNCTION lambda(I) RESULT (RESULT)
  IMPLICIT NONE
  INTEGER I
  COMPLEX*16 d
  COMPLEX*16 diagUpper
  COMPLEX*16 diagMain(0:999)
  COMPLEX*16 diagLower
  COMMON diagUpper, diagMain, diagLower
  RESULT = diagLower/d(I - 1)
END
RECURSIVE COMPLEX*16 FUNCTION d(I) RESULT (RESULT)
  IMPLICIT NONE
  INTEGER I
  COMPLEX*16 lambda
  COMPLEX*16 diagUpper
  COMPLEX*16 diagMain(0:999)
  COMPLEX*16 diagLower
  COMMON diagUpper, diagMain, diagLower
  IF (I .EQ. 0) THEN
    RESULT = diagMain(I)
  ELSE
    RESULT = diagMain(I) - lambda(I)*diagUpper
  END IF
END


